;  ------------------------------------------------------------------

Proc Payroll_Main()

;  The main procedure with the WAIT command

Private
   emp,         ;employee #
   old_pay,     ;pay value in a record before it's changed
   ts,          ;dynamic array with table names
   whs          ;dynamic array with window handles

;  Initialize the desktop

Open_Payroll()

;  The WAIT command

Wait Table
   Proc "Wait_Proc_Payroll"
   Key "F2","Del","Undo","Dos","DosBig"
   Message "MenuSelect"
   Trigger "TouchRecord","PostRecord","DepartField"
EndWait

If sysmode()="CoEdit" then Do_it! Endif
Quit "Wait retval="+strval(retval)

EndProc

;  ------------------------------------------------------------------

Proc Wait_Proc_Payroll(t, e, c)

;  Wait procedure which handles the Payroll data entry process,
;  and passes through most events to Paradox

;  Parameters:
;     t = trigger type
;     e = event record
;     c = cycle number

Private
   err_msg,     ;error message
   k,           ;key code
   mt,          ;menu tag
   new_pay,     ;$ amount from current payroll record
   wh           ;window handle of the form

;  If we are tracing events, write this event to the window

If is_event_trace then
   Wait_Trace(t,e,c)
Endif

;  Determine which event has occurred, and how to process it

Switch

;  ---------- Menu Selection Events ----------

   Case t="EVENT" and e["Type"]="MESSAGE" and e["Message"]="MENUSELECT":
      mt = e["MenuTag"]

;  The "QUIT" menu choice:
;  First simulate an [F2] to terminate co-edit, then quit.

      If mt="Quit" and not isfieldview() then
         If sysmode()="CoEdit" and recordstatus("Modified") then
            e["Type"]     = "KEY"      ;modify the event-bag...
            e["KeyCode"]  = asc("F2")  ;...to look like an [F2] keypress
            e["ScanCode"] = 0
            Wait_Proc_Payroll("EVENT", e, c)       ;process an [F2] event
            If retval<>0 then Return retval Endif  ;it failed
         Endif
         Return 2             ;flag for terminating data entry
      Endif

;  ---------- Key-Press Events ----------

   Case t="EVENT" and e["Type"]="KEY":
      k = e["KeyCode"]
      Switch                  ;determine which key was pressed

;  In field view:  Pass all keys along to Paradox, except illegal ones

         Case isfieldview():                          ;we're in field view
            If k=asc("Dos") or k=asc("DosBig") then   ;Ctrl-O and Alt-O
               Beep Return 1                          ;these keys are illegal
            Endif                                     ;pass others to Paradox

;  The [F2] key:  End the edit session and terminate data entry.
;  If we're sitting on an unposted record, 
;  we must successfully post it, before quitting.
      
         Case k=asc("F2"):
            If sysmode()="CoEdit" and recordstatus("Modified") then
               Wait_Proc_Payroll("POSTRECORD", e, c)  ;force POSTRECORD event
               If retval<>0 then Return retval Endif  ;it failed
            Endif
            Return 2                      ;terminate processing

;  The [Delete] key:  Delete a record.
;  Processing for the Delete key is similar to the PostRecord event.

         Case k=asc("Del"):
            If sysmode()<>"CoEdit" then   ;we're not editing
               Beep Return 1              ;disallow this key
            Endif

            If recordstatus("New") then   ;it's a new record
               Del                        ;so simply delete it
               Return 1                   ;don't let Paradox events occur
            Endif

            Confirm_Del()                          ;ask user for confirmation
            If retval<>0 then Return retval Endif  ;user says to cancel

            If not recordstatus("Locked") then     ;the record is not locked
               LockRecord                          ;so lock it - see note (2)
               If not retval then                  ;couldn't lock it
                  Beep
                  Message errormessage()           ;show error msg to user
                  Return 1                         ;don't proceed
               Endif
               Wait_Proc_Payroll("TOUCHRECORD", e, c)    ;force TOUCHRECORD
               If retval<>0 then Return retval Endif     ;it failed
            Endif

            Before_PostRecord_Payroll(True)        ;do before-post process
            If retval<>0 then Return retval Endif  ;it failed
            Del                                    ;delete the record
            If recordstatus("Locked") then         ;uh-oh, it's still here!
               Failed_PostRecord_Payroll()         ;reverse before-posting
               Beep
               Message "Can't delete this record"
               Return 1                            ;return a failure flag
            Endif
            After_PostRecord_Payroll()             ;do after-post process

            Wait_Proc_Payroll("ARRIVEROW", e, c)   ;force ARRIVEROW
            Wait_Proc_Payroll("ARRIVEFIELD", e, c) ;force ARRIVEFIELD
            
            Return 1             ;don't let Paradox do a second [Delete]

;  [Ctrl-U] (Undo) key:
;     This key is not always allowed; 
;     only changes-in-progress may be undone.   - See note (5)

         Case k=asc("Undo"):
            If sysmode()="CoEdit" 
               and not recordstatus("New") 
               and not recordstatus("Locked")
               and not recordstatus("Modified") then
                  Message "Only changes-in-progress may be undone"
                  Beep 
                  Return 1
            Endif

;  Any other key (such as Ctrl-O or Alt-O):  Illegal

         Otherwise:
            Beep
            Return 1             ;tell Paradox to ignore this key

      EndSwitch

;  ---------- Trigger Events ----------

   Case t="TOUCHRECORD":
      emp     = [Emp. #]         ;save the original employee #
      old_pay = [Pay Amount]     ;save the original pay amount

;  The POSTRECORD event

   Case t="POSTRECORD":

;  Make sure we're sitting on a record that will be posted

      If not recordstatus("Modified") then   ;the record hasn't changed and
         Return 0                            ;so there's nothing to do
      Endif                                  ;see note (1) below

;  Perform validation on the record before it is posted

      Validate_Payroll_Record()              ;perform validation on record
      If retval<>0 then Return retval Endif  ;validation failed

;  Make sure a DEPARTFIELD occurs before the POSTRECORD - see note (3)

      Wait_Proc_Payroll("DEPARTFIELD", e, c) ;force a DEPARTFIELD event
      If retval<>0 then Return retval Endif  ;it failed

;  Perform processing on related tables that is needed before record posting

      Before_PostRecord_Payroll(False)       ;do before-posting processing
      If retval<>0 then Return retval Endif  ;it failed

;  Post the record

      PostRecord NoPost    ;post it

;  If the posting fails, display an error message

      If not retval or recordstatus("Modified") then   ;failed to post
         If errorcode()<>0 then                 ;Paradox gave us a message
            err_msg = errormessage()            ;show it to the user
         Else
            err_msg = "Can't post this record - Duplicate key error"
         Endif

         Failed_PostRecord_Payroll()            ;reverse before-posting work

         Beep
         Message err_msg

         Return 1                               ;return a failure flag
      Endif

;  Perform processing on related tables that is needed before record posting

      After_PostRecord_Payroll()             ;do after-posting processing
      If retval<>0 then Return retval Endif  ;it failed

;  The DEPARTFIELD event on a field requiring validation

   Case t="DEPARTFIELD" and sysmode()="CoEdit" and recordstatus("Modified"):

      Switch

         Case field()="Pay Date":
            If not isblank([]) and dow([])<>"Fri" then
               Beep
               Message "The PAY DATE must be a Friday"
               Return 1
            Endif

         Case field()="Emp. #":     ;see note (4)
            If not recordstatus("New") and []<>emp then
               Beep
               Message "You may not change the EMPLOYEE #"
               [] = emp
               Return 1
            Endif
      EndSwitch

EndSwitch

;  ---------- Successful Completion of Event Handling ----------

Return 0

EndProc

;  ------------------------------------------------------------------

Proc Validate_Payroll_Record()

;  Validate the payroll record before it is posted.

;  This procedure is called before adding or updating a record,
;  but not for deleting one.

;  This procedure returns RETVAL=0 for success, RETVAL=1 for failure.

;  Validate required fields

If isblank([Pay Date]) then
   Beep
   Message "The PAY DATE is required"
   MoveTo [Pay Date]
   Return 1
Endif

If isblank([Emp. #]) then
   Beep
   Message "The EMP. # is required"
   MoveTo [Emp. #]
   Return 1
Endif

If isblank([Pay Amount]) then
   Beep
   Message "The PAY AMOUNT is required"
   MoveTo [Pay Amount]
   Return 1
Endif

Return 0       ;successful completion

EndProc

;  ------------------------------------------------------------------

Proc Before_PostRecord_Payroll(is_deleting)

;  Peform any processing that is needed to prepare related tables,
;  prior to posting the user's current record.
;  Usually, this entails locating and locking records in related tables.

;  This procedure is called before adding, updating, OR DELETING a record.

;  Parameter:
;     is_deleting = TRUE if the user wants to delete the record
;                 = FALSE if the user wants to post the record

;  This procedure returns RETVAL=0 for success, RETVAL=1 for failure.

;  Get the "new" pay amount.

If is_deleting then           ;we plan to delete the current record
   new_pay = 0                ;treat the new pay as zero
Else                          ;we plan to add or update the current record
   new_pay = [Pay Amount]     ;so save the new pay amount
Endif

;  If it's a new record about to be added, 
;  then there was never a TOUCHRECORD event.
;  To compensate, use zero as the old amount and capture the employee # now.

If recordstatus("New") then   ;it's a new record
   old_pay = 0                ;there was no "old" amount
   emp     = [Emp. #]         ;remember the employee #
Endif

;  Locate and lock the related EMPLOYEE record

;  Locate it

wh = GetWindow()     ;remember the current window @
MoveTo "Employee"    ;switch to the employee table
MoveTo [Emp. #]      ;the field to search within
Locate emp           ;look for the employee record

;  Failure to locate it

If not retval then   ;not found
   Beep
   Message "Emp. # "+emp+" not found in the EMPLOYEE table"
   Window Select wh  ;return to the PAYROLL form
   Return 1
Endif

;  Lock it

LockRecord           ;attempt to lock the record
If not retval then   ;could not lock it
   Beep
   Message "Could not lock the EMPLOYEE record for Emp. # "+emp
   Window Select wh  ;return to the PAYROLL form
   Return 1
Endif

;  Return to the PAYROLL record

Window Select wh              ;return to the PAYROLL form

Return 0                      ;successful completion

EndProc

;  ------------------------------------------------------------------

Proc After_PostRecord_Payroll()

;  Complete any processing that is needed on related tables,
;  after the successful posting of the user's current record.
;  Usually, this entails updating and unlocking records which
;  were locked by the "Before_PostRecord" routine.

;  This procedure returns RETVAL=0 for success, RETVAL=1 for failure.
;  However, there should be no reasonable possibility for failure.
;  Thus, RETVAL=0 should be virtually a foregone conclusion.

;  Update the EMPLOYEE record

MoveTo "Employee"                ;switch to the employee table

[YTD Pay] = [YTD Pay] + new_pay - old_pay  ;update the EMPLOYEE record

UnlockRecord                     ;release the locked record
Window Select wh                 ;return to the PAYROLL form

Return 0                         ;successful completion

EndProc

;  ------------------------------------------------------------------

Proc Failed_PostRecord_Payroll()

;  Undo any processing that is needed on related tables,
;  after a failure to post the user's current record.
;  Usually, this entails unlocking records which were locked 
;  by the "Before_PostRecord" routine without updating them.

;  This procedure returns RETVAL=0 for success, RETVAL=1 for failure.
;  However, there should be no reasonable possibility for failure.
;  Thus, RETVAL=0 should be virtually a foregone conclusion.

;  Unlock the EMPLOYEE record

MoveTo "Employee"                ;switch to the employee table
UnlockRecord                     ;release the locked record
Window Select wh                 ;return to the PAYROLL form

Return 0                         ;successful completion

EndProc

;  ------------------------------------------------------------------

Proc Confirm_Del()

;  When the user has pressed the [Delete] key,
;  confirm whether they really want to delete this record.
;  Display a dialog box to the user.

;  This procedure returns RETVAL=0 if the user okays the deletion,
;  or RETVAL=1 if the user does not.

ShowDialog "Confirm Deletion" @ 8,23 Height 8 Width 34
   @ 1,5 ?? "  Do you really want  "
   @ 2,5 ?? "to delete this record?"
   PaintCanvas Attribute syscolor(3) 1,3,2,28
   PushButton @ 4,2  Width 12 "OK" OK
      Value "OK" Tag "OK" To sdv
   PushButton @ 4,17 Width 12 "Cancel" Cancel
      Value "Cancel" Tag "Cancel" To sdv
EndDialog

If not retval then Return 1 Endif      ;Cancel

Return 0                               ;OK

EndProc

;  ------------------------------------------------------------------

Proc Open_Payroll()

;  Perform initialization for the payroll data entry session.
;  Open windows, among other things.

Private
   wb           ;window attributes dynarray

Echo Normal     ;this might be fun to watch

DynArray whs[]                          ;a place to store window handles
DynArray ts[]                           ;a place to store table names
   
DynArray wb[]
wb["HasShadow"] = False
wb["Floating"]  = False

View "Payroll"
Window Handle Image imageno() to wh
whs["Pay"] = wh
ts["Pay"]  = upper(table())
Window Select wh
Window Move wh to 1000,1000             ;push it off the screen
PickForm 1
wh = GetWindow()
whs["PayF"] = wh
Window SetAttributes wh from wb
Window Resize wh to 11,36
Window Move wh to 2,0

View "Employee"
wh = GetWindow()
whs["Emp"] = wh
ts["Emp"]  = upper(table())
Window Resize wh to 7,42
If is_event_trace then
   Window Move wh to 4,37
Else
   Window Move wh to 15,27
Endif

Echo Off

Window Select whs["PayF"]
CoEditKey

Menu_Payroll()

If is_event_trace then
   whs["EvTr"] = evtrace_params["Window Handle"]
Endif

EndProc

;  ------------------------------------------------------------------

Proc Menu_Payroll()

;  Display the pull-down menu in use during payroll data entry.

ShowPullDown
   "Quit":    "All Done":               "Quit"
EndMenu

EndProc

;  ------------------------------------------------------------------

Payroll_Main()

;  ------------------------------------------------------------------

;  THE EVENT MODEL FOR DEPARTING AND POSTING RECORDS

;                                    Which Events Fire?
;                                  ---------------------
;                                  T   D               A
;                                  O   E       P       R
;                                  U   P   D   O   A   R
;                                  C   A   E   S   R   I
;                                  H   R   P   T   R   V
;                                  R   T   A   R   I   E
;                                  E   F   R   E   V   F
;                                  C   I   T   C   E   I
;                                  O   E   R   O   R   E
;                                  R   L   O   R   O   L    Record
;  User Action    Record Status    D   D   W   D   W   D    Disposition
;  -----------    -------------    -   -   -   -   -   -    -----------------
;
;  Move Off       New Blank            DF  DR  PR  AR  AF   deleted
;  the Record     New Blank, but       DF  DR      AR  AF   stays new blank
;                   can't move
;                 New Modified         DF  DR  PR  AR  AF   posted
;                 New Modified,        DF  DR      AR  AF   stays new blank
;                   but can't move
;                 Existing Unlocked    DF  DR      AR  AF
;                 Existing Locked      DF  DR      AR  AF   unlocked
;                   and Unmodified
;                 Existing Modified,   DF  DR      AR  AF   stays locked
;                   but can't move
;                 Existing Modified    DF  DR  PR  AR  AF   posted
;                   no fly-away
;                 Existing Modified    DF  DR  PR  AR  AF   posted
;                   with fly-away
;
;  Alt-L          New Blank                    PR  AR  AF   posts a blank
;                 New Modified                 PR  AR  AF   posted
;                   with fly-away
;                 New Modified                 PR  AR  AF   posted
;                   no fly-away
;                 Existing Unlocked                         no events fire
;                 Existing Locked                           unlocked,
;                   and Unmodified                            no events fire
;                 Existing Modified            PR           posted
;                   no fly-away
;                 Existing Modified            PR  AR  AF   posted
;                   with fly-away              
;
;  Del            New Blank        TR  DF  DR      AR  AF   deleted
;                 New Modified     TR  DF  DR      AR  AF   deleted
;                 Exist. Unlocked  TR  DF  DR      AR  AF   deleted
;                 Exist. Locked    TR  DF  DR      AR  AF   deleted
;                   and Unmodified
;                 Existing Mod.    TR  DF  DR      AR  AF   deleted
;
;  Peculiar event sequences that may require special attention:
;
;  (a)   When you move off a new blank record, the events
;        DEPARTFIELD, DEPARTROW, and POSTRECORD fire, even though
;        Paradox will delete this record, not post it.
;
;  (b)   When you press [ALT-L] on an unposted record, the events
;        DEPARTFIELD nor DEPARTROW do not fire, even though you 
;        might have some critical code there that must execute 
;        before the record is posted, and even though you might 
;        depart from the record in the case of a fly-away.
;
;  (c)   When you press the [Delete] key, a TOUCHRECORD occurs even
;        though you do not intend to change the record.  In fact
;        If you have already begun changing the record and then
;        press [Delete], the TOUCHRECORD event fires twice:  First,
;        when you begin changing the record, and second, when you
;        press [Delete].

;  ------------------------------------------------------------------

;  Notes:

;  (1)   When you exit from a new unmodified record, Paradox fires
;        the POSTRECORD event.  Since Paradox will be deleting
;        this record, it makes sense to exit from the POSTRECORD
;        routine right away, to skip validity checking and other
;        processing.  We are sitting on a new unmodified record
;        when RecordStatus("Modified")=False.

;  (2)   If you press [Del] on a record which you have already
;        begun changing, the record will already be locked and
;        the TOUCHRECORD event will already have fired, capturing
;        the record as it looked originally.
;           However, if you press [Del] on a record which has not
;        been changed, no TOUCHRECORD has taken place, so we need
;        to manufacture one.
;           Locking a record before deleting it is important,
;        because it gives us a good way to determine whether the
;        deletion is successful.  If it is not successful, we'll
;        remain on this locked record.

;  (3)   If the user posts a record by pressing some non-movement
;        posting key, such as [Alt-L] or [F2], then Paradox will
;        not call any DEPART... events.  Since the DEPARTFIELD
;        event contains a validation test, we must force it to
;        take place.

;  (4)   The program does not allow you to change the employee # in
;        an existing payroll record.  If it did, it would have to
;        simultaneously lock, update, and unlock two records in the
;        EMPLOYEE table at the same time.  Paradox does not support
;        locking two records in the same table image.  There is a
;        way to simultaneously lock two records in a table, but its
;        not attractive:  You view the same table twice, giving you
;        two separate images to move through and lock records in.
;        I've chosen not to implement this technique in this sample
;        program.

;  (5)   Paradox allows you to undo a change even when you've moved
;        off the record and posted it.  Unfortunately, it gives the 
;        PAL program no indication of what change its undoing, so 
;        there's no way we can update related tables (the EMPLOYEE
;        table, in this case).  Therefore, we must disable the 
;        capability.  On the other hand, there's no reason for us to 
;        disable the ability to undo changes to a record that you
;        haven't yet posted.

;  ------------------------------------------------------------------
