;  ------------------------------------------------------------------

Proc Rcd_Stat()

;  Returns the status of a record as a text string

;  This procedure relies heavily on the RECORDSTATUS() function.

;  RECORDSTATUS() Values:
;        "New"   "Locked"   "Modified"   "KeyViol"    This function returns:
;        -----   --------   ----------   ---------    ----------------------
;        TRUE      any        FALSE        any          "New/Blank"
;        TRUE      any        TRUE         TRUE         "New/KeyViol"
;        TRUE      TRUE       TRUE         FALSE      * "New/Locked"
;        TRUE      FALSE      TRUE         FALSE        "New/Modified"
;        FALSE     any        any          TRUE         "Exist/KeyViol"
;        FALSE     any        TRUE         FALSE        "Exist/Modified"
;        FALSE     TRUE       FALSE        FALSE        "Exist/Locked"
;        FALSE     FALSE      FALSE        FALSE        "Exist/Untouched"
;
;  If SysMode()<>"CoEdit"                               "Not CoEdit"

;  * In the table above, items marked with "*" are supposedly impossible.

;  Make sure we're in CoEdit mode

If sysmode()<>"CoEdit" then Return "Not CoEdit" Endif

;  "New" record

If recordstatus("New") then                  ;a new record
   If not recordstatus("Modified") then      ;new but not modified
      Return "New/Blank"                     ;new blank record
   Endif
   If recordstatus("KeyViol") then           ;new with key-viol
      Return "New/KeyViol"
   Endif
   If recordstatus("Locked") then            ;new and locked (impossible?)
      Return "New/Locked"
   Endif
   Return "New/Modified"                     ;new and modified
Endif

;  "Existing" record

If recordstatus("KeyViol") then              ;existing with key-viol
   Return "Exist/KeyViol"
Endif
If recordstatus("Modified") then             ;existing and modified
   Return "Exist/Modified"
Endif
If recordstatus("Locked") then               ;existing and locked, unmodified
   Return "Exist/Locked"
Endif
Return "Exist/Untouched"                     ;existing, unlocked, untouched

EndProc

;  ------------------------------------------------------------------

Proc Key_Desc(n)

;  Function to convert an Ascii numeric key-code, n,
;  Into a text string describing the key.
;  Returns a null if it can't identify the key.

Switch
   Case isblank(n):            Return ""
   Case n=0:                   Return "Ctrl-Break"
   Case n>255 or n<-132:       Return ""
   Case n>=32 and n<=126:      Return "\"" + chr(n) + "\""
   Case n=9:                   Return "Tab"
   Case n=13:                  Return "Enter"
   Case n>=1 and n<=26:        Return "Ctrl-"+chr(n+64)
   Case n=27:                  Return "Esc"
   Case n=127:                 Return "Ctrl-Backspace"
   Case n>=128:                Return "\"" + chr(n) + "\""
   Case n>0:                   Return "Ascii "+strval(n)
   Case n<=-59 and n>=-68:     Return "F"+strval(-58-n)
   Case n<=-84 and n>=-93:     Return "Shift-F"+strval(-83-n)
   Case n<=-94 and n>=-103:    Return "Ctrl-F"+strval(-93-n)
   Case n<=-104 and n>=-113:   Return "Alt-F"+strval(-103-n)
   Case n<=-120 and n>=-128:   Return "Alt-"+strval(-119-n)
   Case n=-129:                Return "Alt-0"
   Case n=-15:                 Return "Shift-Tab"
   Case n=-16:                 Return "Alt-Q"
   Case n=-17:                 Return "Alt-W"
   Case n=-18:                 Return "Alt-E"
   Case n=-19:                 Return "Alt-R"
   Case n=-20:                 Return "Alt-T"
   Case n=-21:                 Return "Alt-Y"
   Case n=-22:                 Return "Alt-U"
   Case n=-23:                 Return "Alt-I"
   Case n=-24:                 Return "Alt-O"
   Case n=-25:                 Return "Alt-P"
   Case n=-30:                 Return "Alt-A"
   Case n=-31:                 Return "Alt-S"
   Case n=-32:                 Return "Alt-D"
   Case n=-33:                 Return "Alt-F"
   Case n=-34:                 Return "Alt-G"
   Case n=-35:                 Return "Alt-H"
   Case n=-36:                 Return "Alt-J"
   Case n=-37:                 Return "Alt-K"
   Case n=-38:                 Return "Alt-L"
   Case n=-44:                 Return "Alt-Z"
   Case n=-45:                 Return "Alt-X"
   Case n=-46:                 Return "Alt-C"
   Case n=-47:                 Return "Alt-V"
   Case n=-48:                 Return "Alt-B"
   Case n=-49:                 Return "Alt-N"
   Case n=-50:                 Return "Alt-M"
   Case n=-71:                 Return "Home"
   Case n=-72:                 Return "Up"
   Case n=-73:                 Return "PgUp"
   Case n=-75:                 Return "Left"
   Case n=-77:                 Return "Right"
   Case n=-79:                 Return "End"
   Case n=-80:                 Return "Down"
   Case n=-81:                 Return "PgDn"
   Case n=-82:                 Return "Ins"
   Case n=-83:                 Return "Del"
   Case n=-114:                Return "Ctrl-PrtSc"
   Case n=-115:                Return "Ctrl-Left"
   Case n=-116:                Return "Ctrl-Right"
   Case n=-117:                Return "Ctrl-End"
   Case n=-118:                Return "Ctrl-PgDn"
   Case n=-119:                Return "Ctrl-Home"
   Case n=-130:                Return "Alt-Minus"
   Case n=-131:                Return "Alt="
   Case n=-132:                Return "Ctrl-Pgup"
EndSwitch

Return ""

EndProc

;  ------------------------------------------------------------------

